# Arcanea Ruby 2D Engine (ARE)
#
# Author:: Josef Smolka (mailto:maurel@gmail.com)
# Copyright:: Copyright (c) 2007 Josef Smolka
# Licence:: MIT Licence (http://www.opensource.org/licenses/mit-license.php)

require 'aresprite.rb'
require 'arevector.rb'

class AREAnimation < ARESprite
  attr_reader :started, :running

  public

  # Prepare sprite for usage
  def initialize(parentSurface, hwSurface = false)
    super(parentSurface, hwSurface)

    @initialDelay = 0
    @delay = 0
    @repeat = false
    @running = false
    @started = false
    @startTime = 0
    @lastTime = 0
  end

  # Set animation's properties
  def setAnimation(initialDelay, delay, repeat = false)
    @initialDelay = initialDelay
    @delay = delay
    @repeat = repeat
  end

  # Start animation
  def start()
    @startTime = SDL.getTicks
    @started = true
  end

  # Stop animation
  def stop()
    @running = false
    @started = false
  end

  # Render animation
  def render(surface, x, y)
    if @started then
      if (@startTime + @initialDelay) <= SDL.getTicks and not @running then
        @running = true
      end

      self.stop if (@displayFrame == @frames.size - 1) and not @repeat
      if @running then
        self.nextFrame
        super
      end
    end
  end
end

class AREAnimationVec < AREAnimation
  attr_accessor :position

  def initialize(parentSurface, hwSurface = false)
    super(parentSurface, hwSurface)
    @position = AREVector.new
  end

  def render(surface)
    super(surface, @position.x, @position.y)
  end
end
