require 'structure.rb'

TILE_H = 36
TILE_W = 36
THEME_GREEN = 0
THEME_BLUE = 1
THEME_DEFAULT = THEME_GREEN
RANKS = ["Governor", "Great Defender", "Hero of Mars", "Ruler of Mars"]
MAP_S = 6
MARS_MAX_HP = 1000
PIRATES_MSGS = [
  "Enemy is approaching!", "Pirates !? Pirates!", "COMM: Resistance is futile!",
  "COMM: You'll be assimilated!", "COMM: Hey hou!", 
  "Hyperspace jump signature detected.", "COMM: Are we there yet?!",
  "COMM: We are just passing by.", "Hostile ships detected."
]

class Interface
  attr_accessor :mars_year, :mars_credits, :mars_research, :mars_people
  attr_accessor :mars_hp, :mars_bonuses, :mars_message
  attr_reader :mars_map

  def initialize(scr)
    @screen = scr
    @bmp_mars = SDL::Surface.load('./gfx/mars.bmp')
    @bmp_research = SDL::Surface.load('./gfx/research.bmp')
    @bmp_credits = SDL::Surface.load('./gfx/credits.bmp')
    @bmp_people = SDL::Surface.load('./gfx/people.bmp')

    @clr_dark = [@screen.format.mapRGB(0x38, 0x6f, 0x3c), @screen.format.mapRGB(0x3b, 0x80, 0x7e)]
    @clr_light = [@screen.format.mapRGB(0x0f, 0x90, 0x1c), @screen.format.mapRGB(0x2c, 0xb3, 0xae)]
    @clr_font = [[0x38, 0x6f, 0x3c], [0x3b, 0x80, 0x7e]]

    # set default color theme
    @color_theme = THEME_DEFAULT
    # selected structures
    @selected_structure = -1
    @msover_structure = -1
    @sell_selected = false
    # player status
    @mars_credits = 200
    @mars_people = 20
    @mars_research = 0
    @mars_year = 2400
    @mars_rank = RANKS[0]
    @mars_message = ""
    @mars_hp = MARS_MAX_HP
    @mars_bonuses = [0, 0, 0]

    # load structures for menu
    spritemap = SDL::Surface.load('./gfx/structures.bmp')
    @structures_civil = Array.new
    @structures_army = Array.new
    5.times do |i|
      ns = SDL::Surface.new(SDL::HWSURFACE, TILE_W, TILE_H, @screen)
      SDL.blitSurface(spritemap, i * TILE_W, 0, TILE_W, TILE_H, ns, 0, 0)
      ns.setColorKey(SDL::SRCCOLORKEY, ns[0,0])
      @structures_civil << ns.displayFormat
    end
    4.times do |i|
      ns = SDL::Surface.new(SDL::HWSURFACE, TILE_W, TILE_H, @screen)
      SDL.blitSurface(spritemap, i * TILE_W, TILE_H, TILE_W, TILE_H, ns, 0, 0)
      ns.setColorKey(SDL::SRCCOLORKEY, ns[0,0])
      @structures_army << ns.displayFormat
    end

    @bmp_credits.setColorKey(SDL::SRCCOLORKEY, @bmp_credits[0,0])
    @bmp_research.setColorKey(SDL::SRCCOLORKEY, @bmp_research[0,0])
    @bmp_people.setColorKey(SDL::SRCCOLORKEY, @bmp_people[0,0])
    @bmp_credits = @bmp_credits.displayFormat
    @bmp_research = @bmp_research.displayFormat
    @bmp_people = @bmp_people.displayFormat

    # mars map
    @mars_map = Array.new(MAP_S)
    MAP_S.times do |i|
      @mars_map[i] = Array.new(MAP_S)
    end
  end

  def draw(font, fontsmall)
    # draw Mars background
    @screen.put(@bmp_mars, 386, 113)
    # draw main frame
    @screen.lock
    @screen.drawLine(20, 10, 20, 470, @clr_dark[@color_theme])
    @screen.drawLine(700, 10, 700, 470, @clr_dark[@color_theme])
    @screen.drawLine(10, 50, 710, 50, @clr_dark[@color_theme])
    @screen.drawLine(10, 440, 710, 440, @clr_dark[@color_theme])
    @screen.unlock

    # draw net on Mars
    @screen.lock
    (MAP_S + 1).times do |i|
      @screen.drawLine(460, 190 + i * TILE_H, 460 + TILE_W * MAP_S, 190 + i * TILE_H, @clr_light[@color_theme])
      @screen.drawLine(460 + i * TILE_W, 190, 460 + i * TILE_W, 190 + MAP_S * TILE_H, @clr_light[@color_theme])
    end
    @screen.drawFilledRectAlpha(460, 190, TILE_W * MAP_S, MAP_S * TILE_H, @clr_dark[@color_theme], 60)
    @screen.unlock

    # draw messages box
    @screen.lock
    @screen.drawFilledRectAlpha(25, 445, 640, 20, @clr_dark[@color_theme], 60)
    @screen.drawRect(25, 445, 640, 20, @clr_dark[@color_theme])
    @screen.unlock
    font.drawSolidUTF8(@screen, "> #{@mars_message}", 30, 447, *@clr_font[@color_theme])

    # draw color selector
    @screen.lock
    @screen.drawFilledRectAlpha(5, 10, 10, 10, @clr_dark[THEME_GREEN], 80 + ((@color_theme + 1) % 2) * 80)
    @screen.drawRect(5, 10, 10, 10, @clr_dark[THEME_GREEN])
    @screen.drawFilledRectAlpha(5, 25, 10, 10, @clr_dark[THEME_BLUE], 80 + (@color_theme % 2) * 80)
    @screen.drawRect(5, 25, 10, 10, @clr_dark[THEME_BLUE])
    @screen.unlock

    # draw mars hp bar
    contract = (@mars_hp.to_f / MARS_MAX_HP.to_f)
    w = (390 * contract).round
    @screen.lock
    @screen.drawFilledRectAlpha(700, 50 + 390 - w, 10, w, @clr_light[@color_theme], 80)
    @screen.drawLine(710, 50, 710, 440, @clr_dark[@color_theme])
    @screen.unlock

    # draw civil structures in menu
    tw = TILE_W + 5
    5.times do |i|
      aa = 0
      aa = 1 if i == @selected_structure
      @screen.lock
      @screen.drawFilledRectAlpha(25 + i * tw, 10, TILE_W, TILE_H, @clr_dark[@color_theme], 60 + 60 * aa)
      @screen.drawRect(25 + i * tw, 10, TILE_W, TILE_H, @clr_dark[@color_theme])
      @screen.unlock
      @screen.put(@structures_civil[i], 25 + i * tw, 10)
    end
    4.times do |i|
      aa = 0
      aa = 1 if i + 5 == @selected_structure
      @screen.lock
      @screen.drawFilledRectAlpha(230 + i * tw, 10, TILE_W, TILE_H, @clr_dark[@color_theme], 60 + 60 * aa)
      @screen.drawRect(230 + i * tw, 10, TILE_W, TILE_H, @clr_dark[@color_theme])
      @screen.unlock
      @screen.put(@structures_army[i], 230 + i * tw, 10)
    end

    # draw recycle box
    aa = 0
    aa = 1 if @sell_selected
    @screen.lock
    @screen.drawFilledRectAlpha(660, 10, TILE_W, TILE_H, @clr_dark[@color_theme], 60 + 60 * aa)
    @screen.drawRect(660, 10, TILE_W, TILE_H, @clr_dark[@color_theme])
    @screen.unlock
    font.drawSolidUTF8(@screen, "$", 673, 20, *@clr_font[@color_theme])

    if @msover_structure != -1
      w = (TILE_W + 5) * 3.5
      h = TILE_H * 2.5
      x = 25 + @msover_structure * (TILE_W + 5)
      y = 50
      @screen.lock
      @screen.drawFilledRectAlpha(x, y, w, h, @clr_dark[@color_theme], 60)
      @screen.unlock
      if @msover_structure < 5 then
        fontsmall.drawSolidUTF8(@screen, "#{STR_NAMES[@msover_structure]}", x + 5, y + 5, *@clr_font[@color_theme])
      else
        fontsmall.drawSolidUTF8(@screen, "#{STR_NAMES[@msover_structure]} (D:#{STR_DAMAGES[@msover_structure]} R:#{STR_RANGES[@msover_structure]})", x + 5, y + 5, *@clr_font[@color_theme])
      end
      @screen.put(@bmp_credits, x + 5, y + 20)
      @screen.put(@bmp_research, x + 5, y + 40)
      @screen.put(@bmp_people, x + 5, y + 60)
      font.drawSolidUTF8(@screen, "#{STR_COSTS_CREDITS[@msover_structure]} (#{STR_BONUSES_CREDITS[@msover_structure]})", x + 30, y + 20, *@clr_font[@color_theme])
      font.drawSolidUTF8(@screen, "#{STR_COSTS_RESEARCH[@msover_structure]} (#{STR_BONUSES_RESEARCH[@msover_structure]})", x + 30, y + 40, *@clr_font[@color_theme])
      font.drawSolidUTF8(@screen, "#{STR_COSTS_PEOPLE[@msover_structure]} (#{STR_BONUSES_PEOPLE[@msover_structure]})", x + 30, y + 60, *@clr_font[@color_theme])
    end

    # draw player status
    @screen.put(@bmp_credits, 400, 10)
    @screen.put(@bmp_research, 500, 10)
    @screen.put(@bmp_people, 580, 10)
    fontsmall.drawSolidUTF8(@screen, "#{@mars_credits.to_s} (#{@mars_bonuses[0].to_s})", 425, 13, *@clr_font[@color_theme])
    fontsmall.drawSolidUTF8(@screen, "#{@mars_research.to_s} (#{@mars_bonuses[1].to_s})", 525, 13, *@clr_font[@color_theme])
    fontsmall.drawSolidUTF8(@screen, "#{@mars_people.to_s} (#{@mars_bonuses[2].to_s})", 605, 13, *@clr_font[@color_theme])
    font.drawSolidUTF8(@screen, "YEAR: #{@mars_year.to_s}", 400, 30, *@clr_font[@color_theme])
    font.drawSolidUTF8(@screen, "RANK: #{@mars_rank}", 500, 30, *@clr_font[@color_theme])

    # draw structures on mars
    @mars_bonuses = [0, 0, 0]
    MAP_S.times do |i|
      MAP_S.times do |j|
        x = 460 + i * TILE_W
        y = 190 + j * TILE_H
        s = @mars_map[i][j]
        if not s.nil? then
          s.draw(x, y)
          # recompute bonuses
          @mars_bonuses[0] += s.bonuses[0]
          @mars_bonuses[1] += s.bonuses[1]
          @mars_bonuses[2] += s.bonuses[2]
        end
      end
    end
  end

  def drawFPS(fps, font)
    font.drawSolidUTF8(@screen, "#{fps.to_s}", 670, 447, *@clr_font[@color_theme])
  end

  def clicked()
    ms_x = SDL::Mouse.state[0]
    ms_y = SDL::Mouse.state[1]

    if ms_x >= 5 and ms_y >= 10 and ms_x <= 15 and ms_y <= 20 then
      @color_theme = THEME_GREEN
    end
    if ms_x >= 5 and ms_y >= 25 and ms_x <= 15 and ms_y <= 35 then
      @color_theme = THEME_BLUE
    end

    MAP_S.times do |i|
      MAP_S.times do |j|
        x = 460 + i * TILE_W
        y = 190 + j * TILE_H
        xw = x + TILE_W
        yh = y + TILE_H
        if ms_x >= x and ms_y >= y and ms_x <= xw and ms_y <= yh then
          if @selected_structure != -1
            s = Structure.new(@selected_structure, @screen, x, y, TILE_W, TILE_H)
            c = s.costs
            if c[0] <= @mars_credits and c[1] <= @mars_research and c[2] <= @mars_people
              @mars_map[i][j] = s
              @mars_credits -= c[0]
              @mars_research -= c[1]
              @mars_people -= c[2]
            else
              @mars_message = "You don't have enough resources for this type of building."
            end
          end
          if @sell_selected
            s = @mars_map[i][j]
            if not s.nil? then
              @mars_credits += (STR_COSTS_CREDITS[s.type] / 2).round
              @mars_map[i][j] = nil
            end
          end
        end
      end
    end

    if ms_x >= 660 and ms_y >= 10 and ms_x <= 660 + TILE_W and ms_y <= 10 + TILE_H then
      @sell_selected = true
    else
      @sell_selected = false
    end

    selected = false
    tw = TILE_W + 5
    9.times do |i|
      if ms_x >= 25 + i * tw and ms_y >= 10 and ms_x <= 25 + i * tw + TILE_W and ms_y <= 10 + TILE_H then
        @selected_structure = i
        selected = true
      end
    end
    @selected_structure = -1 if not selected
  end

  def mousemove()
    ms_x = SDL::Mouse.state[0]
    ms_y = SDL::Mouse.state[1]

    selected = false
    tw = TILE_W + 5
    9.times do |i|
      if ms_x >= 25 + i * tw and ms_y >= 10 and ms_x <= 25 + i * tw + TILE_W and ms_y <= 10 + TILE_H then
        @msover_structure = i
        selected = true
      end
    end
    @msover_structure = -1 if not selected
  end
end
